--- wine-current/dlls/dinput8/dinput8_main.c	2006-05-09 11:29:51.000000000 -0700
+++ wine/dlls/dinput8/dinput8_main.c	2006-05-15 12:00:12.000000000 -0700
@@ -51,8 +51,41 @@ static void UnlockModule(void)
  *	DirectInput8Create (DINPUT8.@)
  */
 HRESULT WINAPI DirectInput8Create(HINSTANCE hinst, DWORD dwVersion, REFIID riid, LPVOID *ppDI, LPUNKNOWN punkOuter) {
-    /* TODO: Create the interface using CoCreateInstance as that's what windows does too and check if the version number >= 0x800 */
-    return DirectInputCreateEx(hinst, dwVersion, riid, ppDI, punkOuter);
+    HRESULT hr;
+
+    TRACE("hInst (%p), dwVersion: %d, riid (%s), punkOuter (%p))\n", hinst, dwVersion, debugstr_guid(riid), punkOuter);
+
+    /* The specified version needs to be dinput8 (0x800) or higher */
+    if(dwVersion < 0x800)
+        return DIERR_OLDDIRECTINPUTVERSION;
+
+    if( !(IsEqualGUID(&IID_IDirectInput8A, riid) || IsEqualGUID(&IID_IDirectInput8W, riid) || IsEqualGUID(&IID_IUnknown, riid)) )
+        return DIERR_INVALIDPARAM;
+
+    CoInitialize(NULL);
+
+    hr = CoCreateInstance( &CLSID_DirectInput8, punkOuter, CLSCTX_INPROC_SERVER, riid, ppDI);
+    if(FAILED(hr)) {
+        ERR("CoCreateInstance failed with hr = %ld\n", hr);
+        return DIERR_INVALIDPARAM;
+    }
+
+    CoUninitialize();
+
+    /* When aggregation is used (punkOuter!=NULL) the application needs to manually call Initialize. */
+    if(punkOuter == NULL && IsEqualGUID(&IID_IDirectInput8A, riid)) {
+        LPDIRECTINPUTA DI = NULL;
+        DirectInputCreateA(hinst, dwVersion, &DI, punkOuter);
+        IDirectInput8_Initialize(DI, hinst, dwVersion);
+    }
+
+    if(punkOuter == NULL && IsEqualGUID(&IID_IDirectInput8W, riid)) {
+        LPDIRECTINPUTW DI = NULL;
+        DirectInputCreateW(hinst, dwVersion, &DI, punkOuter);
+        IDirectInput8_Initialize(DI, hinst, dwVersion);
+    }
+
+    return S_OK;
 }
 
 /*******************************************************************************
@@ -84,8 +117,8 @@ static HRESULT WINAPI DI8CF_CreateInstan
     IClassFactoryImpl *This = (IClassFactoryImpl *)iface;
 
     TRACE("(%p)->(%p,%s,%p)\n",This,pOuter,debugstr_guid(riid),ppobj);
-    if( IsEqualGUID( &IID_IDirectInput8A, riid ) || IsEqualGUID( &IID_IDirectInput8W, riid ) ) {
-        return DirectInput8Create(0, DIRECTINPUT_VERSION, riid, ppobj, pOuter);
+    if( IsEqualGUID( &IID_IDirectInput8A, riid ) || IsEqualGUID( &IID_IDirectInput8W, riid ) || IsEqualGUID( &IID_IUnknown, riid )) {
+        return DirectInputCreateEx(0, DIRECTINPUT_VERSION, riid, ppobj, pOuter);
     }
 
     ERR("(%p,%p,%s,%p) Interface not found!\n",This,pOuter,debugstr_guid(riid),ppobj);    
